using System;
using System.IO;
using System.Collections.Generic;
using Gdk;
using System.Runtime.Serialization.Formatters.Binary;

namespace positive
{
	public static class Cache
	{
		private static Dictionary<string,object> _cache = new Dictionary<string, object> ();
		private static Dictionary<string,Func<object,byte[]>> _serializer = new Dictionary<string,Func<object,byte[]>> ();
		private static Dictionary<string,Func<byte[],object>> _deserializer = new Dictionary<string, Func<byte[], object>> ();
		private const string CACHE_NAME = "cache";

		static Cache ()
		{

		}

		public static void Store<T> (T Data, string key)
		{
			if (_cache.ContainsKey (key))
				_cache [key] = Data;
			else
				_cache.Add (key, Data);
		}

		public static T Get<T> (string key)
		{
			if (_cache.ContainsKey (key))
				return (T)_cache [key];
			return default(T);
		}

		public static bool ContainsKey (string key)
		{
			return _cache.ContainsKey (key);
		}

		public static void SaveCache ()
		{
			try {
				using (BinaryWriter bw = new BinaryWriter(File.Create(CACHE_NAME))) {
					bw.Write (_cache.Count);
					using (MemoryStream ms= new MemoryStream()) {
						BinaryFormatter bf = new BinaryFormatter ();
						foreach (var item in _cache) {
							if (item.Value != null) {
								if (_serializer.ContainsKey (item.Value.GetType ().FullName)) {
									var ser = _serializer [item.Value.GetType ().FullName];
									byte[] buffer = ser.Invoke (item.Value);
									bw.Write (item.Key);
									bw.Write (item.Value.GetType ().FullName);
									bw.Write (buffer.Length);
									bw.Write (buffer);
								} else {
									bf.Serialize (ms, item.Value);
									byte[] buffer = ms.GetBuffer ();
									bw.Write (item.Key);
									bw.Write (item.Value.GetType ().FullName);
									bw.Write (buffer.Length);
									bw.Write (buffer);
									ms.Seek (0, SeekOrigin.Begin);
								}
							}
						}
					}
				}
			} catch {
				if (File.Exists (CACHE_NAME))
					File.Delete (CACHE_NAME);
			}
		}

		public static void LoadCache ()
		{
			try {
				if (!File.Exists (CACHE_NAME))
					return;

				_cache = new Dictionary<string, object> ();
				using (FileStream fs = File.OpenRead(CACHE_NAME)) {
					using (BinaryReader br = new BinaryReader(fs)) {
						using (MemoryStream ms= new MemoryStream()) {
							BinaryFormatter bf = new BinaryFormatter ();
							int count = br.ReadInt32 ();
						
							for (int i=0; i<count; i++) {
								string Key = br.ReadString ();
								string typename = br.ReadString ();
								int length = br.ReadInt32 ();
								byte[] buffer = br.ReadBytes (length);

								if (_deserializer.ContainsKey (typename)) {
									var ser = _deserializer [typename];
									object item = ser.Invoke (buffer);
									_cache.Add (Key, item);
								} else {
									ms.Write (buffer, 0, length);
									ms.Seek (0, SeekOrigin.Begin);
									object item = bf.Deserialize (ms);
									_cache.Add (Key, item);
								}

							}
						}
					}
				}
			} catch {
				if (File.Exists (CACHE_NAME))
					File.Delete (CACHE_NAME);
			}
		}

		public static void AddCustomSerializer<T> (Func<object,byte[]> serializer, Func<byte[],object> deserializer)
		{
			_serializer.Add (typeof(T).FullName, serializer);
			_deserializer.Add (typeof(T).FullName, deserializer);
		}
	}
}

