using System;
using Google.Apis.Plus.v1;
using System.Collections.Generic;
using Google.Apis.Plus.v1.Data;
using System.IO;
using Gdk;
using System.Net;

namespace positive
{
	public static class Helper
	{
		private const string API_KEY ="AIzaSyBM8P9NvrOyinp_xuKsjdeLsDgJg5-9i4Y";
		private const string PERSONS_FILENAME ="persons";
		private static object _lock;

		public static PlusService Service;
		public static List<Person> FollowingPersons 
		{
			get{
				return new List<Person>( FollowingPersonsIdMap.Values);
			}
		}
		public static Dictionary<string,Person> FollowingPersonsIdMap = new Dictionary<string, Person>();

		static Helper ()
		{
			Service = new PlusService ();
			Service.Key = API_KEY;
			_lock = new object();
		}
		
		public static void OpenFollowingPersons ()
		{
			if(!File.Exists (PERSONS_FILENAME)) return;
			using(BinaryReader br = new BinaryReader(File.OpenRead (PERSONS_FILENAME)))
			{
				int count = br.ReadInt32 ();
				for(int i=0;i<count;i++)
				{
					Person p = new Person();
					p.Id = br.ReadString ();
					p.Url = br.ReadString ();
					p.AboutMe = br.ReadString ();
					p.Birthday = br.ReadString ();
					p.DisplayName = br.ReadString ();
					p.Nickname = br.ReadString ();
					p.Tagline = br.ReadString ();

					p.Name = new Person.NameData();
					p.Name.FamilyName = br.ReadString ();
					p.Name.MiddleName = br.ReadString ();
					p.Name.GivenName = br.ReadString ();
					p.Name.HonorificPrefix = br.ReadString ();
					p.Name.HonorificSuffix = br.ReadString ();
					p.Name.Formatted = br.ReadString ();

					p.Image = new Person.ImageData();
					p.Image.Url = br.ReadString ();
					FollowingPersonsIdMap.Add (p.Id,p);
				}	
			}
		}
		public static void SaveFollowingPersons ()
		{
			using(BinaryWriter bw = new BinaryWriter(File.Create (PERSONS_FILENAME)))
			{
				bw.Write (FollowingPersons.Count);
				foreach (var item in FollowingPersons) {
					bw.Write (item.Id == null ? string.Empty:item.Id); 
					bw.Write(item.Url==null ? string.Empty:item.Url);
					bw.Write(item.AboutMe==null ? string.Empty:item.AboutMe);
					bw.Write(item.Birthday==null ? string.Empty:item.Birthday);
					bw.Write(item.DisplayName==null ? string.Empty:item.DisplayName);
					bw.Write(item.Nickname==null ? string.Empty:item.Nickname);
					bw.Write(item.Tagline==null ? string.Empty:item.Tagline);
					if(item.Name==null)item.Name = new Person.NameData();
					bw.Write(item.Name.FamilyName==null ? string.Empty:item.Name.FamilyName);
					bw.Write(item.Name.MiddleName==null ? string.Empty:item.Name.MiddleName);
					bw.Write(item.Name.GivenName==null ? string.Empty:item.Name.GivenName);
					bw.Write(item.Name.HonorificPrefix==null ? string.Empty:item.Name.HonorificPrefix);
					bw.Write(item.Name.HonorificSuffix==null ? string.Empty:item.Name.HonorificSuffix);
					bw.Write(item.Name.Formatted==null ? string.Empty:item.Name.Formatted);
					if(item.Image == null) item.Image = new Person.ImageData();
					bw.Write(item.Image.Url==null ? string.Empty:item.Image.Url);
				}	
			}
		}

		public static IList<Activity> GetActivities ()
		{
			List<Activity> retlist = new List<Activity>();
			foreach (var item in Helper.FollowingPersons) {
				var req = Helper.Service.Activities.List (item.Id, Google.Apis.Plus.v1.ActivitiesResource.Collection.Public);
				var activityFeed = req.Fetch ();
				foreach (var activity in activityFeed.Items) {
					retlist.Add (activity);
				}
			}
			retlist.Sort ( (x, y) => {
				DateTime pub1 = DateTime.Parse (x.Published);
				DateTime pub2 = DateTime.Parse (y.Published);
				return pub1.CompareTo (pub2);
			});
			return retlist;
		}

		public static Pixbuf GetImage(string url)
		{
			return GetImage (url,0);
		}
		public static Pixbuf GetImage (string url, int size)
		{
			lock(_lock)
			{
				if(size>0)
				{
					int index = url.IndexOf ("?sz=");
					if(index>0)
					{
						url=url.Substring (0,index);
					}
					url=url+"?sz="+size;
				}
				if(Cache.ContainsKey (url))
						return Cache.Get<Pixbuf>(url);
				Pixbuf image = DownloadImage (url);
				Cache.Store<Pixbuf>(image,url);
				return image;
			}
		}
		public static Pixbuf DownloadImage(string url)
		{
			try{
			url = System.Web.HttpUtility.UrlDecode(url);
			var req = HttpWebRequest.Create (url);
			HttpWebResponse resp = (HttpWebResponse)req.GetResponse();
			Stream s = resp.GetResponseStream ();
			Pixbuf pbuf =new Pixbuf(s);
			return pbuf;
			}
			catch (WebException x)
			{
				Console.WriteLine ("Helper.DownloadImage WebException: "+x.Message);
				return null;
			}
		}

		public static void InitCache()
		{
			//Pixbuf caching
			Cache.AddCustomSerializer<Pixbuf>(x=>{
				Pixbuf pb = (Pixbuf)x;
				byte[] buffer = pb.SaveToBuffer ("png");
				return buffer;
			},x=>{
				byte[] buffer = (byte[])x;
				Pixbuf pb = new Pixbuf(buffer);
				return pb;
			});
		}
	}


}

