using System;
using System.IO;
using Gdk;
using Google.Apis.Plus.v1.Data;

namespace positive
{
	public partial class SearchPerson : Gtk.Window
	{
	    Gtk.ListStore myModel = new Gtk.ListStore(typeof(Person),typeof(bool));

		public SearchPerson () : 
				base(Gtk.WindowType.Toplevel)
		{
			this.Build ();

			Gtk.TreeViewColumn checkCol = new Gtk.TreeViewColumn ();
			checkCol.Title="Selected";
			Gtk.CellRendererToggle toggleRenderer = new Gtk.CellRendererToggle();
			toggleRenderer.Activatable = true;
			toggleRenderer.Toggled+= HandleToggled;
			checkCol.PackStart (toggleRenderer,true);
			checkCol.SetCellDataFunc (toggleRenderer,RenderToggle);

			Gtk.TreeViewColumn picCol = new Gtk.TreeViewColumn ();
			picCol.Title="Picture";
			Gtk.CellRendererPixbuf pixbufRenderer = new Gtk.CellRendererPixbuf();
			picCol.PackStart (pixbufRenderer,true);
			picCol.SetCellDataFunc (pixbufRenderer,RenderPersonPicture);

			Gtk.TreeViewColumn nameCol = new Gtk.TreeViewColumn ();
			nameCol.Title="Name";
			Gtk.CellRendererText textRenderer = new Gtk.CellRendererText();
			nameCol.PackStart (textRenderer,true);
			nameCol.SetCellDataFunc (textRenderer,RenderPersonName);


			PersonTreeView.AppendColumn (checkCol);
			PersonTreeView.AppendColumn (picCol);
			PersonTreeView.AppendColumn (nameCol);
			PersonTreeView.Model = myModel;
		}

		void HandleToggled (object o, Gtk.ToggledArgs args)
		{
			Gtk.TreeIter iter;
			myModel.GetIter (out iter, new Gtk.TreePath( args.Path));
			Person p = (Person) myModel.GetValue (iter,0);
			bool selected = !(bool) myModel.GetValue (iter,1);
			myModel.SetValue (iter,1,selected);
			if(!selected && Helper.FollowingPersonsIdMap.ContainsKey (p.Id))
				Helper.FollowingPersonsIdMap.Remove (p.Id);
			if(selected && !Helper.FollowingPersonsIdMap.ContainsKey (p.Id))
				Helper.FollowingPersonsIdMap.Add (p.Id,p);
			Helper.SaveFollowingPersons ();
		}
			
		private void RenderPersonPicture (Gtk.TreeViewColumn column, Gtk.CellRenderer cell, Gtk.TreeModel model, Gtk.TreeIter iter)
		{
			Person p = (Person) model.GetValue (iter,0);
			if(p==null)return;
			var image = Helper.GetImage (p.Image.Url,32);
			if(image!=null)
				(cell as Gtk.CellRendererPixbuf).Pixbuf = image;
		}
		private void RenderPersonName (Gtk.TreeViewColumn column, Gtk.CellRenderer cell, Gtk.TreeModel model, Gtk.TreeIter iter)
		{
			Person p = (Person) model.GetValue (iter,0);
			if(p==null)return;
			(cell as Gtk.CellRendererText).Text = p.DisplayName;
		}
		private void RenderToggle (Gtk.TreeViewColumn column, Gtk.CellRenderer cell, Gtk.TreeModel model, Gtk.TreeIter iter)
		{
			bool b = (bool) model.GetValue (iter,1);
			(cell as Gtk.CellRendererToggle).Active = b;
		}

		protected void OnButtonSearchClicked (object sender, EventArgs e)
		{
			string name = NameEntry.Text;
			if(string.IsNullOrEmpty (name))
				return;

			myModel.Clear ();
			NameEntry.Sensitive = false;
			buttonSearch.Sensitive = false;
			
			var search = Helper.Service.People.Search (name);
			search.BeginFetch(ar=>{
				lock(myModel){
				var persons = search.EndFetch (ar);
				if(persons.Error!=null)
					{
						var md =new Gtk.MessageDialog(this, Gtk.DialogFlags.Modal,
						                              Gtk.MessageType.Error, Gtk.ButtonsType.Close,
						                              "Error");
						md.Text = "Error!";
						md.SecondaryText = persons.Error.Message;
						md.Run ();
						md.Destroy();
					}
				
				foreach(var item in persons.Items)
				{					
					myModel.AppendValues (item,Helper.FollowingPersonsIdMap.ContainsKey (item.Id));
					}
					NameEntry.Sensitive = true;
					buttonSearch.Sensitive = true;
				}
			},null);
		}
		protected void OnNameEntryKeyReleaseEvent (object o, Gtk.KeyReleaseEventArgs args)
		{
			if(args.Event.Key == Gdk.Key.Return)
				OnButtonSearchClicked (o,new EventArgs());
		}

		protected void OnButtonOkClicked (object sender, System.EventArgs e)
		{
			this.Hide ();
		}
		protected void OnButtonShowPersonsClicked (object sender, EventArgs e)
		{
			myModel.Clear ();
			foreach (var item in Helper.FollowingPersons) {
				myModel.AppendValues (item,true);
			}
		}
	}
}

